from timemachines.skaters.allskaters import SKATERS, skater_from_name
from timemachines.skating import prior, posterior, residuals
from timemachines.skatertools.visualization.priorplot import prior_plot, prior_plot_exogenous